package controller;

import java.util.ResourceBundle;

import javax.xml.bind.DatatypeConverter;

import com.jfoenix.controls.JFXComboBox;
import com.jfoenix.controls.JFXTextArea;

import javafx.concurrent.Task;
import javafx.concurrent.WorkerStateEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import main.Main;
import model.CipherModel;

public class CipherController {

	@FXML
	private JFXTextArea textAreaPlain, textAreaMcEliece, textAreaNied, textAreaMcElieceD, textAreaNiedD;

	private CipherModel model;

	@FXML
	private Label keycreated;

	private ResourceBundle resourceBundle;

	@FXML
	private JFXComboBox<Integer> combo;

	public CipherController() {

	}

	@FXML
	public void initialize() {
		resourceBundle = ResourceBundle.getBundle(Main.locale);
		combo.getItems().setAll(1024, 2048, 4096, 8373911);
	}

	@FXML
	public void generateKeys() {
		try {
			if (combo.getSelectionModel().getSelectedItem() == 8373911) {
				keycreated.setText(resourceBundle.getString("cipher.impossible.key"));
			} else {
				keycreated.setText(resourceBundle.getString("cipher.generate.keys"));
				model = new CipherModel();
				int n = combo.getSelectionModel().getSelectedItem();

				Task<Long> task = new Task<Long>() {

					@Override
					protected Long call() throws Exception {
						long in = System.currentTimeMillis();
						model.init(n);
						long out = System.currentTimeMillis();

						return (out - in);
					}

				};

				task.setOnSucceeded(new EventHandler<WorkerStateEvent>() {
					@Override
					public void handle(WorkerStateEvent t) {
						long result = task.getValue();
						keycreated.setText(
								resourceBundle.getString("cipher.generate.keys.completed") + ": " + result + " ms");
					}
				});
				task.setOnFailed(new EventHandler<WorkerStateEvent>() {
					@Override
					public void handle(WorkerStateEvent t) {

						keycreated.setText(resourceBundle.getString("cipher.generate.keys.failed"));
					}
				});

				Thread th = new Thread(task);
				th.setDaemon(false);
				th.start();

			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@FXML
	public void encrypt() {

		if (model != null) {

			String plainText = textAreaPlain.getText();

			byte[] cipher = null;

			try {
				cipher = model.encryptMcEliece(plainText.getBytes());
			} catch (Exception e) {
				e.printStackTrace();
			}

			textAreaMcEliece.setText(DatatypeConverter.printHexBinary(cipher));

			String decrypted = "";
			try {
				decrypted = model.decryptMcEliece(cipher);
			} catch (Exception e) {
				e.printStackTrace();
			}
			textAreaMcElieceD.setText(new String(decrypted));

			byte[] cipher2 = null;
			try {
				cipher2 = model.encryptNied(plainText.getBytes());
			} catch (Exception e) {
				e.printStackTrace();
			}

			textAreaNied.setText(DatatypeConverter.printHexBinary(cipher2));

			String decrypted2 = "";
			try {
				decrypted2 = model.decryptNied(cipher2);
			} catch (Exception e) {
				e.printStackTrace();
			}
			textAreaNiedD.setText(new String(decrypted2));
		}

	}

}
