package controller;

import java.util.ResourceBundle;

import com.jfoenix.controls.JFXSlider;

import de.flexiprovider.common.math.codingtheory.PolynomialGF2mSmallM;
import de.flexiprovider.common.math.linearalgebra.GF2Vector;
import javafx.concurrent.Task;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import main.Main;
import model.McElieceGoppaModel;

public class McElieceGoppaController {

	private McElieceGoppaModel model;

	@FXML
	private TextArea textAreaS, textAreaG, textAreaP, textAreaGP;

	@FXML
	private TextArea textAreaX, textAreaGP2, textAreaE, textAreaE2, textAreaY;

	@FXML
	private TextArea textAreaYP, textAreaSyn, textAreaMSG, textAreaPI, textAreaY2, textAreaSI, textAreaYPP,
			textAreaPoly, textAreaPoly2, textAreaELP;

	@FXML
	private Label alice1, bob1, alice2;

	@FXML
	private Button sbutton, gbutton, pbutton, pkbutton, mbutton, ebutton, cbutton, ypbutton, synbutton, dbutton,
			syndtablebutton;

	@FXML
	private Label pklabel, ylabel, declabel, pilabel, silabel;

	@FXML
	private JFXSlider slider;

	private ResourceBundle resourceBundle;

	public McElieceGoppaController() {
		model = new McElieceGoppaModel();
	}

	@FXML
	public void initialize() {
		resourceBundle = ResourceBundle.getBundle(Main.locale);
	}

	// ####################################################
	// Controladores de Alice para generar la clave publica
	// ####################################################

	@FXML
	public void generateS() {
		model.generateScramblerMatrix();
		alice1.setText(resourceBundle.getString("mceliece.hamming.generateS"));
		sbutton.getStyleClass().add("addOk");

	}

	@FXML
	public void generateG() {
		model.generateLinearCodeMatrices();
		alice1.setText(resourceBundle.getString("mceliece.hamming.generateG"));
		gbutton.getStyleClass().add("addOk");
	}

	@FXML
	public void generateP() {
		if (model.getGeneratorMatrix() != null) {
			model.generatePermutationMatrix();
			alice1.setText(resourceBundle.getString("mceliece.hamming.generateP"));
			pbutton.getStyleClass().add("addOk");
		} else {
			alice1.setText(resourceBundle.getString("mceliece.goppa.error.generateGbeforeP"));
		}
	}

	@FXML
	public void generatePublicKey() {

		String sb = checkConditionsForPublicKey();

		try {
			model.getPublicKey(model.getScramblerMatrix(), model.getGeneratorMatrix(), model.getPermutationMatrix());

			alice1.setText(resourceBundle.getString("mceliece.hamming.generateGP"));
			textAreaGP.setText(model.printMatrixToGUI(model.getPublicKeyMatrix()));
			pkbutton.getStyleClass().add("addOk");
		} catch (Exception e) {
			alice1.setText(sb.toString());
		}

	}

	private String checkConditionsForPublicKey() {
		StringBuilder sb = new StringBuilder();

		sb.append(resourceBundle.getString("mceliece.hamming.error.prefix"));

		boolean s = model.getScramblerMatrix() == null;
		boolean g = model.getGeneratorMatrix() == null;
		boolean p = model.getPermutationMatrix() == null;

		if (s && g && p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.SGP"));
		} else if (s && !g && p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.SP"));
		} else if (s && g && !p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.SG"));
		} else if (s && !g && !p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.S"));
		} else if (!s && g && p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.GP"));
		} else if (!s && !g && p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.P"));
		} else if (!s && g && !p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.G"));
		}

		return sb.toString();

	}

	// ####################################################
	// Controladores de Bob para generar el mensaje cifrado
	// ####################################################

	@FXML
	public void generateX() {
		model.generatePlainTextMessage();
		textAreaX.setText(model.printMatrixToGUI(model.getPlaintext()));

		model.setError(null);
		model.setCiphertext(null);
		model.setUnpermutedCiphertext(null);
		model.setDecodedText(null);
		model.setDecryptedMessage(null);
		model.setSyndrome(null);

		textAreaE.setText("");
		textAreaY.setText("");
	}

	@FXML
	public void generateE() {
		model.generateVectorError();
		textAreaE.setText(model.printMatrixToGUI(model.getError()));
	}

	@FXML
	public void encrypt() {
		if (model.getPublicKeyMatrix() != null) {
			model.encrypt();
			textAreaY.setText(model.printMatrixToGUI(model.getCiphertext()));
		} else {
			alice1.setText(resourceBundle.getString("mceliece.goppa.error.missing.PK"));
		}
	}

	// #######################################################
	// Controladores de Alice para generar el texto descifrado
	// #######################################################

	@FXML
	public void startAnimation() {
		try {
			
			textAreaPoly.setText("");
			textAreaPoly2.setText("");
			textAreaELP.setText("");
			textAreaSyn.setText("");
			textAreaE2.setText("");
			textAreaYPP.setText("");
			textAreaMSG.setText("");
			
			
			Task<Void> task = new Task<Void>() {

				private PolynomialGF2mSmallM syndrom, tt, tau;
				private PolynomialGF2mSmallM a, b, a2, b2, xb2, a2plusXb2;
				private int headCoeff, invHeadCoeff, eval;
				private PolynomialGF2mSmallM elp;

				@Override
				protected Void call() throws Exception {
					model.getCipherWithInversePermutation();
					Thread.sleep((long) slider.getValue());
					model.getSyndromeFromCiphertext();
					textAreaSyn.setText(model.printMatrixToGUI(model.getSyndrome()));
					Thread.sleep((long) slider.getValue());
					pattersonDecoding();
					textAreaE2.setText(model.printMatrixToGUI(model.gf2VectorToSimpleMatrix(model.getZ())));
					Thread.sleep((long) slider.getValue());
					model.correctError();
					textAreaYPP.setText(model.printMatrixToGUI(model.getDecodedText()));
					Thread.sleep((long) slider.getValue());
					model.decrypt();
					textAreaMSG.setText(model.printMatrixToGUI(model.getDecryptedMessage()));

					return null;
				}

				private void pattersonDecoding() throws Exception {
					int n = 1 << model.getField().getDegree();

					// the error vector
					GF2Vector errors = new GF2Vector(n);
					textAreaPoly.appendText("Patterson algorithm\n");
					textAreaPoly.appendText("\n");
					textAreaPoly.appendText(model.getField().toString() + "\n");
					textAreaPoly.appendText("\n");
					Thread.sleep((long) slider.getValue());
					textAreaPoly.appendText("SqrtRootMatrix: " + "\n");
					textAreaPoly.appendText("\n");

					for (PolynomialGF2mSmallM s : model.getSqRootMatrix()) {
						textAreaPoly.appendText(s.toString() + "\n");
					}
					Thread.sleep((long) slider.getValue());

					// if the syndrome vector is zero, the error vector is also zero
					if (!model.getGF2Syndrome().isZero()) {
						// convert syndrome vector to polynomial over GF(2^m)
						syndrom = new PolynomialGF2mSmallM(
								model.getGF2Syndrome().toExtensionFieldVector(model.getField()));
						textAreaPoly2.appendText("Polynomial syndrome: " + syndrom + "\n");
						textAreaPoly2.appendText("\n");

						Thread.sleep((long) slider.getValue());

						// compute T = syndrome^-1 mod gp
						tt = syndrom.modInverse(model.getGp());
						textAreaPoly2.appendText("T = syndrome^-1 mod gp: " + tt + "\n");
						textAreaPoly2.appendText("\n");
						Thread.sleep((long) slider.getValue());
						// compute tau = sqRoot(T + X) mod gp
						tau = (tt.addMonomial(1)).modSquareRootMatrix(model.getSqRootMatrix());
						textAreaPoly2.appendText("tau = sqrt(T+X) mod gp: " + tau + "\n");
						textAreaPoly2.appendText("\n");
						Thread.sleep((long) slider.getValue());

						// compute polynomials a and b satisfying a + b*tau = 0 mod gp
						PolynomialGF2mSmallM[] ab = tau.modPolynomialToFracton(model.getGp());

						a = ab[0];
						b = ab[1];

						// compute the polynomial a^2 + X*b^2
						a2 = a.multiply(a);
						b2 = b.multiply(b);
						xb2 = b2.multWithMonomial(1);
						a2plusXb2 = a2.add(xb2);
						textAreaPoly2.appendText("a^2 + X*b^2: " + a2plusXb2 + "\n");
						textAreaPoly2.appendText("\n");

						Thread.sleep((long) slider.getValue());
						// normalize a^2 + X*b^2 to obtain the error locator polynomial
						headCoeff = a2plusXb2.getHeadCoefficient();
						textAreaPoly2.appendText("Head Coefficient: " + headCoeff + "\n");
						textAreaPoly2.appendText("\n");
						Thread.sleep((long) slider.getValue());
						invHeadCoeff = model.getField().inverse(headCoeff);
						textAreaPoly2.appendText("Inv. Head Coefficient: " + invHeadCoeff + "\n");
						textAreaPoly2.appendText("\n");
						Thread.sleep((long) slider.getValue());
						elp = a2plusXb2.multWithElement(invHeadCoeff);

						textAreaPoly2.appendText("Error Locator Polynomial: " + elp + "\n");
						textAreaPoly2.appendText("\n");

						Thread.sleep((long) slider.getValue());
						// for all elements i of GF(2^m)
						for (int i = 0; i < n; i++) {
							// evaluate the error locator polynomial at i
							eval = elp.evaluateAt(i);
							textAreaELP.appendText("Evaluating at " + i + ": " + eval + "\n");
							Thread.sleep((long) slider.getValue());
							// if polynomial evaluates to zero
							if (eval == 0) {
								// set the i-th coefficient of the error vector
								errors.setBit(i);
							}
						}
					}

					model.setZ(errors);

				}

			};
	
			Thread th = new Thread(task);
			th.setDaemon(false);
			th.start();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@FXML
	public void generateSyn() {
		if (model.getParityCheckMatrix() != null && model.getUnpermutedCiphertext() != null) {
			model.getSyndromeFromCiphertext();
			model.pattersonDecoding();
			model.correctError();
			alice2.setText(resourceBundle.getString("mceliece.hamming.generateSyn"));
			textAreaSyn.setText(model.printMatrixToGUI(model.getSyndrome()));
			textAreaYPP.setText(model.printMatrixToGUI(model.getDecodedText()));
		} else {
			if (model.getUnpermutedCiphertext() == null && model.getParityCheckMatrix() != null) {
				alice2.setText(resourceBundle.getString("mceliece.hamming.error.missing.Y"));
			} else {
				alice2.setText(resourceBundle.getString("mceliece.hamming.error.missing.SYN"));
			}
		}
	}

	@FXML
	public void decrypt() {
		if (model.getDecodedText() != null) {
			model.decrypt();
			alice2.setText(resourceBundle.getString("mceliece.hamming.decrypt"));
			textAreaMSG.setText(model.printMatrixToGUI(model.getDecryptedMessage()));
		} else {
			alice2.setText(resourceBundle.getString("mceliece.hamming.error.missing.YPP"));
		}
	}

}
