package controller;

import java.io.IOException;
import java.util.ResourceBundle;

import com.jfoenix.controls.JFXDecorator;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.image.Image;
import javafx.stage.Stage;
import main.Main;
import model.McElieceHammingModel;

public class McElieceHammingController {

	private McElieceHammingModel model;

	@FXML
	private TextArea textAreaS, textAreaG, textAreaP, textAreaGP;

	@FXML
	private TextArea textAreaX, textAreaGP2, textAreaE, textAreaY;

	@FXML
	private TextArea textAreaYP, textAreaSyn, textAreaMSG, textAreaPI, textAreaY2, textAreaSI, textAreaYPP;

	@FXML
	private Label alice1, bob1, alice2;

	@FXML
	private Button sbutton, gbutton, pbutton, pkbutton, mbutton, ebutton, cbutton, ypbutton, synbutton, dbutton,
			syndtablebutton;

	@FXML
	private Label pklabel, ylabel, declabel, pilabel, silabel;

	private ResourceBundle resourceBundle;

	public McElieceHammingController() {
		model = new McElieceHammingModel();
	}

	@FXML
	public void initialize() {
		resourceBundle = ResourceBundle.getBundle(Main.locale);
	}

	// ####################################################
	// Controladores de Alice para generar la clave publica
	// ####################################################

	@FXML
	public void generateS() {
		model.generateScramblerMatrix();
		alice1.setText(resourceBundle.getString("mceliece.hamming.generateS"));
		textAreaS.setText(model.printMatrixToGUI(model.getScramblerMatrix()));
		textAreaSI.setText(model.printMatrixToGUI(model.getInverseScramblerMatrix()));

	}

	@FXML
	public void generateG() {
		model.generateLinearCodeMatrices();
		alice1.setText(resourceBundle.getString("mceliece.hamming.generateG"));
		textAreaG.setText(model.printMatrixToGUI(model.getGeneratorMatrix()));
	}

	@FXML
	public void generateP() {
		model.generatePermutationMatrix();
		alice1.setText(resourceBundle.getString("mceliece.hamming.generateP"));
		textAreaP.setText(model.printMatrixToGUI(model.getPermutationMatrix()));
		textAreaPI.setText(model.printMatrixToGUI(model.getInversePermutationMatrix()));
	}

	@FXML
	public void generatePublicKey() {

		String sb = checkConditionsForPublicKey();

		try {
			model.getPublicKey(model.getScramblerMatrix(), model.getGeneratorMatrix(), model.getPermutationMatrix());

			alice1.setText(resourceBundle.getString("mceliece.hamming.generateGP"));
			bob1.setText(resourceBundle.getString("mceliece.hamming.bob.intro"));
			textAreaGP.setText(model.printMatrixToGUI(model.getPublicKeyMatrix()));
			textAreaGP2.setText(model.printMatrixToGUI(model.getPublicKeyMatrix()));
		} catch (Exception e) {
			alice1.setText(sb.toString());
		}

	}

	private String checkConditionsForPublicKey() {
		StringBuilder sb = new StringBuilder();

		sb.append(resourceBundle.getString("mceliece.hamming.error.prefix"));

		boolean s = model.getScramblerMatrix() == null;
		boolean g = model.getGeneratorMatrix() == null;
		boolean p = model.getPermutationMatrix() == null;

		if (s && g && p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.SGP"));
		} else if (s && !g && p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.SP"));
		} else if (s && g && !p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.SG"));
		} else if (s && !g && !p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.S"));
		} else if (!s && g && p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.GP"));
		} else if (!s && !g && p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.P"));
		} else if (!s && g && !p) {
			sb.append(resourceBundle.getString("mceliece.hamming.error.missing.G"));
		}

		return sb.toString();

	}

	// ####################################################
	// Controladores de Bob para generar el mensaje cifrado
	// ####################################################

	@FXML
	public void generateX() {
		model.generatePlainTextMessage();
		bob1.setText(resourceBundle.getString("mceliece.hamming.generateX"));
		textAreaX.setText(model.printMatrixToGUI(model.getPlaintext()));

		model.setError(null);
		model.setCiphertext(null);
		model.setUnpermutedCiphertext(null);
		model.setDecodedText(null);
		model.setDecryptedMessage(null);
		model.setSyndrome(null);

		textAreaE.setText("");
		textAreaY.setText("");
		textAreaYP.setText("");
		textAreaYPP.setText("");
		textAreaY2.setText("");
		textAreaSyn.setText("");
		textAreaMSG.setText("");
	}

	@FXML
	public void generateE() {
		model.generateVectorError();
		bob1.setText(resourceBundle.getString("mceliece.hamming.generateE"));
		textAreaE.setText(model.printMatrixToGUI(model.getError()));
	}

	@FXML
	public void encrypt() {

		if (model.getError() != null) {

			if (model.getPublicKeyMatrix() != null) {

				model.encrypt();
				bob1.setText(resourceBundle.getString("mceliece.hamming.encrypt"));
				alice2.setText(resourceBundle.getString("mceliece.hamming.alice2.intro"));
				textAreaY.setText(model.printMatrixToGUI(model.getCiphertext()));
				textAreaY2.setText(model.printMatrixToGUI(model.getCiphertext()));
			} else {
				bob1.setText(resourceBundle.getString("mceliece.hamming.error.missing.PK"));
			}
		}else{
			bob1.setText(resourceBundle.getString("mceliece.hamming.error.missing.E"));
		}
	}

	// #######################################################
	// Controladores de Alice para generar el texto descifrado
	// #######################################################

	@FXML
	public void generateYP() {
		if (model.getCiphertext() != null) {
			model.getCipherWithInversePermutation();
			alice2.setText(resourceBundle.getString("mceliece.hamming.generateYP"));
			textAreaYP.setText(model.printMatrixToGUI(model.getUnpermutedCiphertext()));
		} else {
			alice2.setText(resourceBundle.getString("mceliece.hamming.error.missing.Y"));
		}
	}

	@FXML
	public void generateSyn() {
		if (model.getParityCheckMatrix() != null && model.getUnpermutedCiphertext() != null) {
			model.getSyndromeFromCiphertext();
			model.correctError();
			alice2.setText(resourceBundle.getString("mceliece.hamming.generateSyn"));
			textAreaSyn.setText(model.printMatrixToGUI(model.getSyndrome()));
			textAreaYPP.setText(model.printMatrixToGUI(model.getDecodedText()));
		} else {
			if (model.getUnpermutedCiphertext() == null && model.getParityCheckMatrix() != null) {
				alice2.setText(resourceBundle.getString("mceliece.hamming.error.missing.Y"));
			} else {
				alice2.setText(resourceBundle.getString("mceliece.hamming.error.missing.SYN"));
			}
		}
	}

	@FXML
	public void decrypt() {
		if (model.getDecodedText() != null) {
			model.decrypt();
			alice2.setText(resourceBundle.getString("mceliece.hamming.decrypt"));
			textAreaMSG.setText(model.printMatrixToGUI(model.getDecryptedMessage()));
		} else {
			alice2.setText(resourceBundle.getString("mceliece.hamming.error.missing.YPP"));
		}
	}

	// ############################
	// Controladores suplementarios
	// ############################

	@FXML
	protected void printSyndromeTable(ActionEvent event) throws IOException {
		if (model.getSyndTable()[0][0] != null) {

			Stage stage = new Stage();

			FXMLLoader loader = new FXMLLoader(getClass().getResource("/views/standardArray.fxml"), resourceBundle);
			StandardArrayController controller = new StandardArrayController();
			loader.setController(controller);
			controller.initData(model.getSyndTable());

			Parent root = loader.load();
			JFXDecorator decorator = new JFXDecorator(stage, root);
			decorator.setCustomMaximize(true);
			Scene scene = new Scene(decorator);
			scene.getStylesheets().add(getClass().getResource("/resources/dark-theme.css").toExternalForm());
			stage.setScene(scene);
			Image a = new Image("resources/codigo-binario-negro.png");
			stage.getIcons().add(a);
			stage.setTitle(resourceBundle.getString("stdarray.title"));
			stage.show();
		}
	}

}
