package controller;

import java.io.IOException;
import java.util.ResourceBundle;

import com.jfoenix.controls.JFXDecorator;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.stage.Stage;
import main.Main;
import model.NiederreiterGoppaModel;

public class NiederreiterGoppaController {

	private NiederreiterGoppaModel model;

	@FXML
	private TextArea textAreaS, textAreaG, textAreaP, textAreaGP;

	@FXML
	private TextArea textAreaX, textAreaGP2, textAreaE, textAreaY;

	@FXML
	private TextArea textAreaYP, textAreaSyn, textAreaMSG, textAreaPI, textAreaY2, textAreaSI, textAreaYPP;

	@FXML
	private Label alice1, bob1, alice2;

	@FXML
	private Button sbutton, gbutton, pbutton, pkbutton, mbutton, ebutton, cbutton, ypbutton, synbutton, dbutton,
			syndtablebutton;

	private ResourceBundle resourceBundle;

	public NiederreiterGoppaController() {
		model = new NiederreiterGoppaModel();
	}

	@FXML
	public void initialize() {
		resourceBundle = ResourceBundle.getBundle(Main.locale);
	}

	// ####################################################
	// Controladores de Alice para generar la clave publica
	// ####################################################

	@FXML
	public void generateS() {
		if (model.getParityCheckMatrix() != null) {

			model.generateScramblerMatrix();
			alice1.setText(resourceBundle.getString("nied.hamming.generateS"));
			textAreaS.setText(model.printMatrixToGUI(model.getScramblerMatrix()));
			textAreaSI.setText(model.printMatrixToGUI(model.getInverseScramblerMatrix()));
		} else {
			alice1.setText(resourceBundle.getString("nied.goppa.error.generateHbeforeS"));
		}

	}

	@FXML
	public void generateH() {
		model.generateLinearCodeMatrices();
		alice1.setText(resourceBundle.getString("nied.hamming.generateH"));
		textAreaG.setText(model.printMatrixToGUI(model.getParityCheckMatrix()));
	}

	@FXML
	public void generateP() {
		if (model.getParityCheckMatrix() != null) {
			model.generatePermutationMatrix();
			alice1.setText(resourceBundle.getString("nied.hamming.generateP"));
			textAreaP.setText(model.printMatrixToGUI(model.getPermutationMatrix()));
			textAreaPI.setText(model.printMatrixToGUI(model.getInversePermutationMatrix()));
		} else {
			alice1.setText(resourceBundle.getString("nied.goppa.error.generateHbeforeP"));
		}
	}

	@FXML
	public void generatePublicKey() {

		String sb = checkConditionsForPublicKey();

		try {

			model.getPublicKey(model.getScramblerMatrix(), model.getParityCheckMatrix(), model.getPermutationMatrix());
			alice1.setText(resourceBundle.getString("nied.hamming.generateHP"));
			bob1.setText(resourceBundle.getString("nied.hamming.bob.intro"));
			textAreaGP.setText(model.printMatrixToGUI(model.getPublicKeyMatrix()));
			textAreaGP2.setText(model.printMatrixToGUI(model.getPublicKeyMatrix()));

		} catch (Exception e) {
			alice1.setText(sb.toString());
		}
	}

	private String checkConditionsForPublicKey() {
		StringBuilder sb = new StringBuilder();

		sb.append(resourceBundle.getString("nied.hamming.error.prefix"));

		boolean s = model.getScramblerMatrix() == null;
		boolean g = model.getParityCheckMatrix() == null;
		boolean p = model.getPermutationMatrix() == null;

		if (s && g && p) {
			sb.append(resourceBundle.getString("nied.hamming.error.missing.SHP"));
		} else if (s && !g && p) {
			sb.append(resourceBundle.getString("nied.hamming.error.missing.SP"));
		} else if (s && g && !p) {
			sb.append(resourceBundle.getString("nied.hamming.error.missing.SH"));
		} else if (s && !g && !p) {
			sb.append(resourceBundle.getString("nied.hamming.error.missing.S"));
		} else if (!s && g && p) {
			sb.append(resourceBundle.getString("nied.hamming.error.missing.HP"));
		} else if (!s && !g && p) {
			sb.append(resourceBundle.getString("nied.hamming.error.missing.P"));
		} else if (!s && g && !p) {
			sb.append(resourceBundle.getString("nied.hamming.error.missing.H"));
		}

		return sb.toString();

	}

	// ####################################################
	// Controladores de Bob para generar el mensaje cifrado
	// ####################################################

	@FXML
	public void generateX() {
		model.generatePlainTextMessage();
		bob1.setText(resourceBundle.getString("nied.hamming.generateX"));
		textAreaX.setText(model.printMatrixToGUI(model.getPlaintext()));

		model.setError(null);
		model.setCiphertext(null);
		model.setUnscrambledCiphertext(null);
		model.setDecodedText(null);
		model.setDecryptedMessage(null);
		model.setSyndrome(null);

		textAreaY.setText("");
		textAreaYP.setText("");
		textAreaYPP.setText("");
		textAreaY2.setText("");
		textAreaMSG.setText("");
	}

	@FXML
	public void encrypt() {
		if (model.getPublicKeyMatrix() != null) {
			model.encrypt();
			bob1.setText(resourceBundle.getString("nied.hamming.encrypt"));
			alice2.setText(resourceBundle.getString("nied.hamming.alice2.intro"));
			textAreaY.setText(model.printMatrixToGUI(model.getCiphertext()));
			textAreaY2.setText(model.printMatrixToGUI(model.getCiphertext()));
		} else {
			bob1.setText(resourceBundle.getString("nied.hamming.error.missing.PK"));
		}
	}

	// #######################################################
	// Controladores de Alice para generar el texto descifrado
	// #######################################################

	@FXML
	public void generateYP() {
		if (model.getCiphertext() != null) {
			model.getCipherWithInverseScramble();
			alice2.setText(resourceBundle.getString("nied.hamming.generateYP"));
			textAreaYP.setText(model.printMatrixToGUI(model.getUnscrambledCiphertext()));
		} else {
			alice2.setText(resourceBundle.getString("nied.hamming.error.missing.Y"));
		}
	}

	@FXML
	public void generateSyn() {
		if (model.getParityCheckMatrix() != null && model.getUnscrambledCiphertext() != null) {
			model.correctError();
			alice2.setText(resourceBundle.getString("nied.hamming.generateSyn"));
			textAreaYPP.setText(model.printMatrixToGUI(model.getDecodedText()));
		} else {
			if (model.getUnscrambledCiphertext() == null && model.getParityCheckMatrix() != null) {
				alice2.setText(resourceBundle.getString("nied.hamming.error.missing.Y"));
			} else {
				alice2.setText(resourceBundle.getString("nied.hamming.error.missing.SYN"));
			}
		}
	}

	@FXML
	public void decrypt() {
		if (model.getDecodedText() != null) {
			model.decrypt();
			alice2.setText(resourceBundle.getString("nied.hamming.decrypt"));
			textAreaMSG.setText(model.printMatrixToGUI(model.getDecryptedMessage()));
		} else {
			alice2.setText(resourceBundle.getString("nied.hamming.error.missing.YPP"));
		}
	}

	// ############################
	// Controladores suplementarios
	// ############################

	@FXML
	protected void printSyndromeTable(ActionEvent event) throws IOException {

		if (model.getSyndTable()[0][0] != null) {

			Stage stage = new Stage();

			FXMLLoader loader = new FXMLLoader(getClass().getResource("/views/standardArray.fxml"), resourceBundle);
			StandardArrayController controller = new StandardArrayController();
			loader.setController(controller);
			controller.initData(model.getSyndTable());

			Parent root = loader.load();
			JFXDecorator decorator = new JFXDecorator(stage, root);
			decorator.setCustomMaximize(true);
			Scene scene = new Scene(decorator);
			scene.getStylesheets().add(getClass().getResource("/resources/dark-theme.css").toExternalForm());
			stage.setScene(scene);
			stage.setTitle(resourceBundle.getString("stdarray.title"));
			stage.show();
		}
	}

}
