package model;

import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.Security;
import java.security.spec.KeySpec;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import javax.crypto.Cipher;

import de.flexiprovider.core.FlexiCoreProvider;
import de.flexiprovider.pqc.FlexiPQCProvider;

public class CipherModel {

	private byte[] encodedPublicKeyMcEliece;
	private byte[] encodedPrivateKeyMcEliece;

	private byte[] encodedPublicKeyNied;
	private byte[] encodedPrivateKeyNied;

	public CipherModel() throws Exception {
		Security.addProvider(new FlexiCoreProvider());
		Security.addProvider(new FlexiPQCProvider());

	}

	public void init(int n) throws NoSuchAlgorithmException, NoSuchProviderException {
		KeyPairGenerator kpg = KeyPairGenerator.getInstance("McEliece", "FlexiPQC");
		kpg.initialize(n);

		KeyPair keyPair = kpg.generateKeyPair();
		encodedPublicKeyMcEliece = keyPair.getPublic().getEncoded();
		encodedPrivateKeyMcEliece = keyPair.getPrivate().getEncoded();

		KeyPairGenerator kpg2 = KeyPairGenerator.getInstance("Niederreiter", "FlexiPQC");
		kpg2.initialize(n);

		KeyPair keyPair2 = kpg2.generateKeyPair();
		encodedPublicKeyNied = keyPair2.getPublic().getEncoded();
		encodedPrivateKeyNied = keyPair2.getPrivate().getEncoded();
	}

	public byte[] encryptMcEliece(byte[] messageBytes) throws Exception {
		KeySpec publicKeySpec = new X509EncodedKeySpec(encodedPublicKeyMcEliece);
		KeyFactory keyFactory = KeyFactory.getInstance("McEliece", "FlexiPQC");
		PublicKey publicKey = keyFactory.generatePublic(publicKeySpec);
		SecureRandom secureRand = new SecureRandom();
		Cipher cipher = Cipher.getInstance("McEliecePKCS", "FlexiPQC");
		cipher.init(Cipher.ENCRYPT_MODE, publicKey, secureRand);
		return cipher.doFinal(messageBytes);
	}

	public String decryptMcEliece(byte[] ciphertextBytes) throws Exception {
		KeySpec privateKeySpec = new PKCS8EncodedKeySpec(encodedPrivateKeyMcEliece);
		KeyFactory keyFactory1 = KeyFactory.getInstance("McEliece", "FlexiPQC");
		PrivateKey privateKey = keyFactory1.generatePrivate(privateKeySpec);
		Cipher cipher2 = Cipher.getInstance("McEliecePKCS");
		cipher2.init(Cipher.DECRYPT_MODE, privateKey);

		byte[] decodMessageBytes = cipher2.doFinal(ciphertextBytes);
		return new String(decodMessageBytes);
	}

	public byte[] encryptNied(byte[] messageBytes) throws Exception {
		KeySpec publicKeySpec = new X509EncodedKeySpec(encodedPublicKeyNied);
		KeyFactory keyFactory = KeyFactory.getInstance("Niederreiter", "FlexiPQC");
		PublicKey publicKey = keyFactory.generatePublic(publicKeySpec);
		SecureRandom sr = new SecureRandom();
		Cipher cipher = Cipher.getInstance("NiederreiterPKCS");
		cipher.init(Cipher.ENCRYPT_MODE, publicKey, sr);
		return cipher.doFinal(messageBytes);
	}

	public String decryptNied(byte[] ciphertextBytes) throws Exception {
		KeySpec privateKeySpec = new PKCS8EncodedKeySpec(encodedPrivateKeyNied);
		KeySpec publicKeySpec = new X509EncodedKeySpec(encodedPublicKeyNied);
		KeyFactory keyFactory = KeyFactory.getInstance("Niederreiter", "FlexiPQC");
		PublicKey publicKey = keyFactory.generatePublic(publicKeySpec);
		PrivateKey privateKey = keyFactory.generatePrivate(privateKeySpec);
		Cipher cipher = Cipher.getInstance("NiederreiterPKCS");
		SecureRandom sr = new SecureRandom();
		cipher.init(Cipher.ENCRYPT_MODE, publicKey, sr);
		cipher.init(Cipher.DECRYPT_MODE, privateKey);
		byte[] messageBytes = cipher.doFinal(ciphertextBytes);
		return new String(messageBytes);
	}

	public static void main(String[] args) throws Exception {

		CipherModel m = new CipherModel();
		m.init(1024);
		mceliece(m);
		nied(m);

	}

	public static void mceliece(CipherModel m) throws Exception {
		String text1 = "holaquetal";

		byte[] cipherbytes = m.encryptMcEliece(text1.getBytes());

		String text2 = m.decryptMcEliece(cipherbytes);

		System.out.println(text2);
	}

	public static void nied(CipherModel m) throws Exception {
		String text1 = "holaquetal";

		byte[] cipherbytes = m.encryptNied(text1.getBytes());

		String text2 = m.decryptNied(cipherbytes);

		System.out.println(text2);
	}

}