package model;

import java.util.Arrays;

import org.ejml.simple.SimpleMatrix;

import de.flexiprovider.api.Registry;
import de.flexiprovider.api.SecureRandom;
import de.flexiprovider.common.math.codingtheory.GF2mField;
import de.flexiprovider.common.math.codingtheory.GoppaCode;
import de.flexiprovider.common.math.codingtheory.GoppaCode.MaMaPe;
import de.flexiprovider.common.math.codingtheory.PolynomialGF2mSmallM;
import de.flexiprovider.common.math.codingtheory.PolynomialRingGF2m;
import de.flexiprovider.common.math.linearalgebra.GF2Matrix;
import de.flexiprovider.common.math.linearalgebra.GF2Vector;
import de.flexiprovider.common.math.linearalgebra.Permutation;
import de.flexiprovider.pqc.ecc.ECCKeyGenParameterSpec;

public class McElieceGoppaModel {

	// #########
	// Variables
	// #########

	private SimpleMatrix scramblerMatrix, generatorMatrix, permutationMatrix, parityCheckMatrix, linearCode,
			iScramblerMatrix, iPermutationMatrix;
	private SimpleMatrix publicKey, plaintext, error, ciphertext;
	private SimpleMatrix unpCiphertext, syn, decoded, decrypted;

	private SecureRandom random;
	private int n, m = 4, t = 2;
	private int fieldPoly;
	private GF2mField field;
	
	public GF2mField getField() {
		return field;
	}

	public void setField(GF2mField field) {
		this.field = field;
	}

	public PolynomialGF2mSmallM getGp() {
		return gp;
	}

	public void setGp(PolynomialGF2mSmallM gp) {
		this.gp = gp;
	}

	public PolynomialGF2mSmallM[] getSqRootMatrix() {
		return sqRootMatrix;
	}

	public void setSqRootMatrix(PolynomialGF2mSmallM[] sqRootMatrix) {
		this.sqRootMatrix = sqRootMatrix;
	}
	
	public GF2Vector getGF2Syndrome(){
		return syndrome;
	}

	public void setGF2Syndrome(GF2Vector syndrome) {
		this.syndrome = syndrome;
	}
	
	public GF2Vector getZ(){
		return z;
	}
	
	public void setZ(GF2Vector errors){
		this.z = errors;
	}

	private PolynomialGF2mSmallM gp;
	private PolynomialRingGF2m ring;
	private PolynomialGF2mSmallM[] sqRootMatrix;
	private GF2Matrix gPrime, h, G, S, SI,shortH,shortG;
	private MaMaPe mmp; 
	private Permutation P1, P2, P, PI;
	private GF2Vector x, c, e,z;
	private GF2Vector mSG,cPInv, syndrome, mVec;
	
	
	private PolynomialGF2mSmallM syndrom, tt, tau;
	private PolynomialGF2mSmallM a,b,a2, b2,xb2,a2plusXb2;
	private int headCoeff,invHeadCoeff,eval;
	private PolynomialGF2mSmallM elp;
	
	int k = 8;

	public McElieceGoppaModel() {

		this.random = (random != null) ? random : Registry.getSecureRandom();

		ECCKeyGenParameterSpec mParams = new ECCKeyGenParameterSpec(m, t);

		m = mParams.getM();
		n = mParams.getN();
		t = mParams.getT();
		
		fieldPoly = mParams.getFieldPoly();
		field = new GF2mField(m, fieldPoly);

		gp = new PolynomialGF2mSmallM(field, t, PolynomialGF2mSmallM.RANDOM_IRREDUCIBLE_POLYNOMIAL, random);
		ring = new PolynomialRingGF2m(field, gp);
		sqRootMatrix = ring.getSquareRootMatrix();
	}

	// ######################
	// Acciones de generacion
	// ######################

	public void generateScramblerMatrix() {
		GF2Matrix[] matrixSandInverse = GF2Matrix.createRandomRegularMatrixAndItsInverse(k, random);
		S = matrixSandInverse[0];
		SI = matrixSandInverse[1];

		scramblerMatrix = gf2matrixToSimpleMatrix(S);
		iScramblerMatrix = gf2matrixToSimpleMatrix(SI);
	}

	public void generateLinearCodeMatrices() {
		h = GoppaCode.createCanonicalCheckMatrix(field, gp);
		mmp = GoppaCode.computeSystematicForm(h, random);
		shortH = mmp.getSecondMatrix();
		shortG = (GF2Matrix) shortH.computeTranspose();
		gPrime = shortG.extendLeftCompactForm();
		k = shortG.getNumRows();

		generatorMatrix = gf2matrixToSimpleMatrix(gPrime);
		parityCheckMatrix = gf2matrixToSimpleMatrix(h);
	}

	public void generatePermutationMatrix() {

		P1 = mmp.getPermutation();
		P2 = new Permutation(n, random);
		P = P1.rightMultiply(P2);
		PI = P.computeInverse();

		permutationMatrix = getMatrixFromPermutation(P2);
		iPermutationMatrix = getMatrixFromPermutation(PI);
		
	}

	public void generatePlainTextMessage() {
		x = new GF2Vector(k);

		for (int i = 0; i < k; i++) {
			int s = random.nextInt(2);
			if (s == 1) {
				x.setBit(i);
			}
		}

		plaintext = gf2VectorToSimpleMatrix(x);
	}

	public void generateVectorError() {
		e = new GF2Vector(n);
		e.setBit(random.nextInt(n));
		error = gf2VectorToSimpleMatrix(e);
	}

	// ############################
	// Funciones de calculo interno
	// ############################

	public void encrypt() {
		c = (GF2Vector) (G.computeTranspose().rightMultiply(x)).add(e);
		ciphertext = gf2VectorToSimpleMatrix(c);
	}

	public void decrypt() {
		mVec = (GF2Vector) SI.leftMultiply(mSG.extractRightVector(k));
		decrypted = gf2VectorToSimpleMatrix(mVec);
	}

	public void correctError() {
		
		mSG = (GF2Vector) (cPInv.add(z)).multiply(P1);
		decoded = gf2VectorToSimpleMatrix(mSG);
	}

	public void getSyndromeFromCiphertext() {
		syndrome = (GF2Vector) h.rightMultiply(cPInv);
		syn = gf2VectorToSimpleMatrix(syndrome);
	}

	public void getCipherWithInversePermutation() {
		cPInv = (GF2Vector) c.multiply(PI);
		unpCiphertext = gf2VectorToSimpleMatrix(cPInv);
	}

	public void getPublicKey(SimpleMatrix sS, SimpleMatrix Gg, SimpleMatrix P) {
		G = (GF2Matrix) (S.rightMultiply(gPrime)).rightMultiply(P2);
		publicKey = gf2matrixToSimpleMatrix(G);
	}

	public SimpleMatrix setMod2(SimpleMatrix s) {
		for (int i = 0; i < s.numRows(); i++) {
			for (int j = 0; j < s.numCols(); j++) {
				if (s.get(i, j) == -1.0) {
					s.set(i, j, 1);
				} else {
					s.set(i, j, s.get(i, j) % 2);
				}
			}
		}
		return s;
	}

	private static boolean isEqual(SimpleMatrix s2, SimpleMatrix m2) {
		try {

			for (int i = 0; i < s2.numRows(); i++) {
				for (int j = 0; j < s2.numCols(); j++) {
					if (s2.get(i, j) != m2.get(i, j)) {
						return false;
					}

				}
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	private SimpleMatrix getMatrixFromVector(String word) {

		double[][] mat = new double[1][word.length()];

		for (int i = 0; i < word.length(); i++) {
			if (word.charAt(i) == '0') {
				mat[0][i] = 0.0;
			} else {
				mat[0][i] = 1.0;
			}
		}

		return new SimpleMatrix(mat);
	}

	public void printMatlabMatrix(SimpleMatrix s) {
		StringBuilder sb = new StringBuilder();
		sb.append("[ ");
		sb.append("\n");
		for (int i = 0; i < s.numRows(); i++) {
			for (int j = 0; j < s.numCols(); j++) {
				int ss = (int) s.get(i, j);
				sb.append(ss + " ");
			}
			sb.append(";");
			sb.append("\n");

		}
		sb.append("]");

		System.out.println(sb.toString());
	}

	public String printMatrixToGUI(SimpleMatrix s) {
		StringBuilder sb = new StringBuilder();

		for (int i = 0; i < s.numRows(); i++) {
			for (int j = 0; j < s.numCols(); j++) {
				int ss = (int) s.get(i, j);
				sb.append(ss + " ");
			}
			sb.append("\n");
		}

		return sb.toString();
	}

	public SimpleMatrix gf2matrixToSimpleMatrix(GF2Matrix gf) {
		SimpleMatrix s = new SimpleMatrix(0, gf.getNumColumns());

		String[] st = gf.toString().split("\n");

		for (String h : st) {
			StringBuilder vector = new StringBuilder(h.replace(" ", "").split(":")[1]);

			s = s.concatRows(getMatrixFromVector(vector.toString()));

		}

		return s;
	}

	public SimpleMatrix gf2VectorToSimpleMatrix(GF2Vector gf) {
		return getMatrixFromVector(gf.toString());
	}
	
	
	public void pattersonDecoding() {
		int n = 1 << field.getDegree();

		// the error vector
		GF2Vector errors = new GF2Vector(n);

		System.out.println("Patterson algorithm");
		System.out.println("Field: "+gp);
		System.out.println("Sqrt: "+Arrays.toString(sqRootMatrix));
		System.out.println("Syndrome: "+syndrome);
		
		// if the syndrome vector is zero, the error vector is also zero
		if (!syndrome.isZero()) {
			// convert syndrome vector to polynomial over GF(2^m)
			syndrom = new PolynomialGF2mSmallM(syndrome.toExtensionFieldVector(field));
			System.out.println("Polynomial syndrome: "+syndrom);
			
			// compute T = syndrome^-1 mod gp
			tt = syndrom.modInverse(gp);
			System.out.println(tt);

			// compute tau = sqRoot(T + X) mod gp
			tau = (tt.addMonomial(1)).modSquareRootMatrix(sqRootMatrix);
			System.out.println(tau);
			
			
			// compute polynomials a and b satisfying a + b*tau = 0 mod gp
			PolynomialGF2mSmallM[] ab = tau.modPolynomialToFracton(gp);

			a = ab[0];
			b = ab[1];
			
			// compute the polynomial a^2 + X*b^2
			a2 = a.multiply(a);
			b2 = b.multiply(b);
			xb2 = b2.multWithMonomial(1);
			a2plusXb2 = a2.add(xb2);

			System.out.println(a2plusXb2);
			
			// normalize a^2 + X*b^2 to obtain the error locator polynomial
			headCoeff = a2plusXb2.getHeadCoefficient();
			System.out.println(headCoeff);
			invHeadCoeff = field.inverse(headCoeff);
			System.out.println(invHeadCoeff);
			elp = a2plusXb2.multWithElement(invHeadCoeff);

			System.out.println(elp);
			
			// for all elements i of GF(2^m)
			for (int i = 0; i < n; i++) {
				// evaluate the error locator polynomial at i
				eval = elp.evaluateAt(i);
				System.out.println(eval);
				// if polynomial evaluates to zero
				if (eval == 0) {
					// set the i-th coefficient of the error vector
					errors.setBit(i);
				}
			}
		}

		z = errors;
		System.out.println(z);
	}

	// #################
	// Getters y setters
	// #################

	public SimpleMatrix getScramblerMatrix() {
		return scramblerMatrix;
	}

	public void setScramblerMatrix(SimpleMatrix s) {
		scramblerMatrix = s;
	}

	public SimpleMatrix getGeneratorMatrix() {
		return generatorMatrix;
	}

	public void setGeneratorMatrix(SimpleMatrix g) {
		generatorMatrix = g;
	}

	public SimpleMatrix getPermutationMatrix() {
		return permutationMatrix;
	}

	public void setPermutationMatrix(SimpleMatrix p) {
		permutationMatrix = p;
	}

	public SimpleMatrix getParityCheckMatrix() {
		return parityCheckMatrix;
	}

	public void setParityCheckMatrix(SimpleMatrix h) {
		parityCheckMatrix = h;
	}

	public SimpleMatrix getLinearCode() {
		return linearCode;
	}

	public void setLinearCode(SimpleMatrix c) {
		linearCode = c;
	}

	public SimpleMatrix getCiphertext() {
		return ciphertext;
	}

	public void setCiphertext(SimpleMatrix y) {
		this.ciphertext = y;
	}

	public SimpleMatrix getPlaintext() {
		return plaintext;
	}

	public void setPlaintext(SimpleMatrix x) {
		this.plaintext = x;
	}

	public SimpleMatrix getError() {
		return error;
	}

	public void setError(SimpleMatrix e) {
		this.error = e;
	}

	public SimpleMatrix getPublicKeyMatrix() {
		return publicKey;
	}

	public void setPublicKeyMatrix(SimpleMatrix pk) {
		publicKey = pk;
	}

	public SimpleMatrix getUnpermutedCiphertext() {
		return unpCiphertext;
	}

	public void setUnpermutedCiphertext(SimpleMatrix yp) {
		this.unpCiphertext = yp;
	}

	public SimpleMatrix getSyndrome() {
		return syn;
	}

	public void setSyndrome(SimpleMatrix syn) {
		this.syn = syn;
	}

	public SimpleMatrix getDecodedText() {
		return decoded;
	}

	public void setDecodedText(SimpleMatrix ypp) {
		this.decoded = ypp;
	}

	public SimpleMatrix getDecryptedMessage() {
		return decrypted;
	}

	public void setDecryptedMessage(SimpleMatrix xx) {
		this.decrypted = xx;
	}

	public SimpleMatrix getInverseScramblerMatrix() {
		return iScramblerMatrix;
	}

	public void setInverseScramblerMatrix(SimpleMatrix sI) {
		iScramblerMatrix = sI;
	}

	public SimpleMatrix getInversePermutationMatrix() {
		return iPermutationMatrix;
	}

	public void setInversePermutationMatrix(SimpleMatrix pI) {
		iPermutationMatrix = pI;
	}

	// ###############
	// Main de pruebas
	// ###############

	public SimpleMatrix getMatrixFromPermutation(Permutation p) {
		SimpleMatrix s = new SimpleMatrix(p.getVector().length, p.getVector().length);

		int count = 0;
		for (int i : p.getVector()) {
			s.set(i, count, 1);
			count++;
		}

		return s;
	}

	public static void main(String[] args) {

		McElieceGoppaModel f = new McElieceGoppaModel();

		// Genero las matrices S, G y P
		f.generateScramblerMatrix();
		f.generateLinearCodeMatrices();
		f.generatePermutationMatrix();
		// Genero la clave publica G' = SGP
		f.getPublicKey(f.getScramblerMatrix(), f.getGeneratorMatrix(), f.getPermutationMatrix());

		for (int i = 0; i < 10; i++) {

			// Genero un mensaje x

			f.generatePlainTextMessage();
			// Genero un vector de error e de tamano t

			f.generateVectorError();

			// Genero un mensaje cifrado y = xG'+e
			f.encrypt();

			// Genero un "despermutado" del mensaje cifrado y' = yP-1

			f.getCipherWithInversePermutation();

			// Calculo el sindrome de y' y lo busco en la tabla de sindromes
			// para corregir el error

			f.getSyndromeFromCiphertext();
			f.pattersonDecoding();
			f.correctError();

			// Genero el mensaje descifrado x = y''(8)S-1

			f.decrypt();

			if (isEqual(f.getPlaintext(), f.getDecryptedMessage())) {
				System.out.println("Bien");
			} else {
				System.out.println("Mal");
			}
		}

	}

	

}
